SystemOrganization addCategory: #'Stack-Exercise'!

Object subclass: #OOStack
	instanceVariableNames: 'elements errorChecker'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Stack-Exercise'!

!OOStack class methodsFor: 'error descriptions' stamp: 'HernanWilkinson 5/7/2012 11:51'!
stackEmptyErrorDescription
	
	^ 'Stack is empty'! !

!OOStack methodsFor: 'as yet unclassified' stamp: 'GrossodelSel 5/14/2012 21:08'!
initialize
	super initialize.
	elements := LinkedList new.
	elements addLast: OOStackFloor new.! !

!OOStack methodsFor: 'as yet unclassified' stamp: 'GrossodelSel 5/14/2012 20:53'!
isEmpty
	"comment stating purpose of message"

	| temporary variable names |
	^(elements size) = 1.! !

!OOStack methodsFor: 'as yet unclassified' stamp: 'GrossodelSel 5/14/2012 21:05'!
pop
	 ^(elements removeLast) getElement.! !

!OOStack methodsFor: 'as yet unclassified' stamp: 'GrossodelSel 5/14/2012 20:51'!
push: anObject
	elements addLast: (OOStackElement initializeForElement: anObject).! !

!OOStack methodsFor: 'as yet unclassified' stamp: 'GrossodelSel 5/14/2012 20:51'!
size
	 ^(elements size) - 1.! !

!OOStack methodsFor: 'as yet unclassified' stamp: 'GrossodelSel 5/14/2012 21:06'!
top
	 ^(elements last) getElement.! !

Object subclass: #OOStackElement
	instanceVariableNames: 'element'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Stack-Exercise'!

!OOStackElement class methodsFor: 'as yet unclassified' stamp: 'GrossodelSel 5/14/2012 20:47'!
initializeForElement: anElement
	| aNewObject |
	aNewObject := OOStackElement new.
	aNewObject initializeForElement: anElement.
	^aNewObject.! !

!OOStackElement methodsFor: 'as yet unclassified' stamp: 'GrossodelSel 5/14/2012 20:49'!
getElement
	^element.! !

!OOStackElement methodsFor: 'as yet unclassified' stamp: 'GrossodelSel 5/14/2012 20:48'!
initializeForElement: anElement
	element := anElement.! !

Object subclass: #OOStackFloor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Stack-Exercise'!

!OOStackFloor methodsFor: 'as yet unclassified' stamp: 'GrossodelSel 5/14/2012 21:05'!
getElement
	"comment stating purpose of message"
	self error: OOStack stackEmptyErrorDescription.! !

TestCase subclass: #OOStackTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Stack-Exercise'!

!OOStackTest methodsFor: 'test' stamp: 'HernanWilkinson 5/7/2012 11:30'!
test01StackShouldBeEmptyWhenCreated

	| stack |
	
	stack := OOStack new.
	
	self assert: stack isEmpty! !

!OOStackTest methodsFor: 'test' stamp: 'HernanWilkinson 5/7/2012 11:30'!
test02PushAddElementsToTheStack

	| stack |
	
	stack := OOStack new.
	stack push: 'Something'.
	
	self deny: stack isEmpty! !

!OOStackTest methodsFor: 'test' stamp: 'HernanWilkinson 5/7/2012 11:31'!
test03PopRemovesElementsFromTheStack

	| stack |
	
	stack := OOStack new.
	stack push: 'Something'.
	stack pop.
	
	self assert: stack isEmpty! !

!OOStackTest methodsFor: 'test' stamp: 'HernanWilkinson 5/8/2012 08:20'!
test04PopReturnsLastPushedObject

	| stack pushedObject |
	
	stack := OOStack new.
	pushedObject := 'Something'.
	stack push: pushedObject.
	
	self assert: stack pop = pushedObject! !

!OOStackTest methodsFor: 'test' stamp: 'HernanWilkinson 5/7/2012 11:33'!
test05StackBehavesLIFO

	| stack firstPushedObject secondPushedObject |
	
	stack := OOStack new.
	firstPushedObject := 'First'.
	secondPushedObject := 'Second'.
	
	stack push: firstPushedObject.
	stack push: secondPushedObject.
	
	self assert: stack pop = secondPushedObject.
	self assert: stack pop = firstPushedObject.
	self assert: stack isEmpty 
	! !

!OOStackTest methodsFor: 'test' stamp: 'HernanWilkinson 5/7/2012 11:35'!
test06TopReturnsLastPushedObject

	| stack pushedObject |
	
	stack := OOStack new.
	pushedObject := 'Something'.
	
	stack push: pushedObject.
	
	self assert: stack top = pushedObject.
	! !

!OOStackTest methodsFor: 'test' stamp: 'HernanWilkinson 5/7/2012 11:36'!
test07TopDoesNotRemoveObjectFromStack

	| stack pushedObject |
	
	stack := OOStack new.
	pushedObject := 'Something'.
	
	stack push: pushedObject.
	
	self assert: stack size = 1.
	stack top.
	self assert: stack size = 1.
	! !

!OOStackTest methodsFor: 'test' stamp: 'HernanWilkinson 5/7/2012 11:38'!
test08CanNotPopWhenThereAreNoObjectsInTheStack

	| stack  |
	
	stack := OOStack new.
	self
		should: [ stack pop ]
		raise: Error
		withExceptionDo: [ :anError |
			self assert: anError messageText = OOStack stackEmptyErrorDescription ]
		
! !

!OOStackTest methodsFor: 'test' stamp: 'HernanWilkinson 5/7/2012 11:39'!
test09CanNotTopWhenThereAreNoObjectsInTheStack

	| stack  |
	
	stack := OOStack new.
	self
		should: [ stack top ]
		raise: Error
		withExceptionDo: [ :anError |
			self assert: anError messageText = OOStack stackEmptyErrorDescription ]
		
! !
